﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Collections.Specialized;
using System.Collections;
using System.Windows.Threading;
using System.Windows;
using System.Collections.ObjectModel;

namespace XamDataGridPerformance
{
    /// <summary>
    /// A dataset representing real-time data
    /// </summary>
    public class TickerBuffer : INotifyCollectionChanged, IEnumerable
    {
        public int WindowSize
        {
            get
            {
                return _windowSize;
            }
            set
            {
                _windowSize = value;
            }
        }

        private IEnumerable _inputSource;
        private int _position = 0;
        private ObservableCollection<Row> _historicalDataSource = new ObservableCollection<Row>();
        private int _windowSize;

        public ObservableCollection<Complex> ComplexDataColl
        {
            get
            {
                return _complexDataColl;
            }
        }
        private ObservableCollection<Complex> _complexDataColl = new ObservableCollection<Complex>();

        /// <summary>
        /// Create a buffer using the data supplied, setting a buffer size of 500 items
        /// </summary>
        /// <param name="inputSource">A dataset to be used as input</param>
        public TickerBuffer(IEnumerable inputSource)
        {
            _inputSource = inputSource;
            _windowSize = 500;
        }

        /// <summary>
        /// Creates a buffer using the data supplied, and sets the buffer size to the specified value
        /// </summary>
        /// <param name="inputSource">A dataset to be used as input</param>
        /// <param name="windowSize">A value to be used for buffer size</param>
        public TickerBuffer(IEnumerable inputSource, int windowSize)
        {
            _inputSource = inputSource;
            _windowSize = windowSize;

            _complexDataColl.Add(new Complex(1, "One"));
            _complexDataColl.Add(new Complex(2, "Two"));
            _complexDataColl.Add(new Complex(3, "Three"));

        }

        public void FillWindow()
        {
            if(_historicalDataSource.Count != _windowSize)
                this.AddRecordsFromHistoricalData(this.WindowSize);
        }

        /// <summary>
        /// Add a specified number of records containing randomly generated data to dataset
        /// </summary>
        /// <param name="recordCount"></param>
        public void Generate(int recordCount)
        {
            Random r = new Random();
            DateTime startDate = new DateTime(1990, 1, 1, 0, 0, 0);



            for (int i = 0; i < recordCount; ++i)
            {
                double baseValue = r.Next(100);
                double offsetValue = r.Next(5);
                _historicalDataSource.Add(new Row(startDate.AddSeconds(1).ToShortDateString(),baseValue, baseValue + offsetValue, baseValue - offsetValue, baseValue + 2*offsetValue, 0, _complexDataColl[0]));
            }

            if (CollectionChanged != null)
            {
                CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
            }
        }

        /// <summary>
        /// Add a specified number of records from historical dataset to dataset
        /// </summary>
        /// <param name="recordCount"></param>
        public void AddRecordsFromHistoricalData(int recordCount)
        {
            _historicalDataSource.Clear();

            //add records from historical data
            for (int i = 0; i < recordCount; ++i)
            {
                _historicalDataSource.Add((_inputSource as IList)[(_position) % (_inputSource as IList).Count] as Row);

                ++_position;
            }

            if (CollectionChanged != null)
            {
                CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
            }
        }

        /// <summary>
        /// Add a specified number of records from a historical dataset to this dataset, starting from a specified position
        /// </summary>
        /// <param name="recordCount"></param>
        public void AddRecordsFromHistoricalData(int recordCount, int position)
        {
            _position = position;
            //add records from historical data
            for (int i = 0; i < recordCount; ++i)
            {
                _historicalDataSource.Add((_inputSource as IList)[(_position) % (_inputSource as IList).Count] as Row);

                ++_position;
            }

            //remove old records from the beginning of the window according to window size
            while (_historicalDataSource.Count > _windowSize)
                _historicalDataSource.RemoveAt(0);

            if (CollectionChanged != null)
            {
                CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
            }
        }



        #region IEnumerable Members

        public IEnumerator GetEnumerator()
        {
            return _historicalDataSource.GetEnumerator();
        }

        #endregion

        #region INotifyCollectionChanged Members

        public event NotifyCollectionChangedEventHandler CollectionChanged;

        #endregion
    }

    public class Complex : INotifyPropertyChanged
    {
        public Complex(Int32 id, String displayName)
        {
            _id = id;
            _displayName = displayName;
        }

        public Complex()
        {
            _id = 0;
            _displayName = String.Empty;
        }

        public Int32 Id
        {
            get
            {
                return _id;
            }
            set
            {
                if ( _id != value )
                {
                    _id = value;
                    RaisePropertyChanged("Id");
                }
            }
        }
        private Int32 _id;

        public String DisplayName
        {
            get
            {
                return _displayName;
            }
            set
            {
                if ( !_displayName.Equals(value) )
                {
                    _displayName = value;
                }
            }
        }
        private String _displayName;

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;
        private void RaisePropertyChanged(string propertyName)
        {
            if ( PropertyChanged != null )
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion

    }

    /// <summary>
    /// Row data schema
    /// </summary>
    public class Row : INotifyPropertyChanged
    {
        public static Int32 counter = 1;

        public Row(string date, double open, double high, double low, double close, double volume, Complex complex)
        {
            Date = DateTime.ParseExact(date, "M/d/yyyy", System.Globalization.CultureInfo.InvariantCulture);
            column1 = open;
            _complex = complex;
            column46 = counter++;
        }


        public DateTime Date { get { return column0; } set { column0 = value; RaisePropertyChanged("Date"); } }
        private DateTime column0;

        private double column1;
        private Int32 column46;

        public double Column1 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column1"); } }

        public double Column2 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column2"); } }

        public double Column3 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column3"); } }

        public double Column4 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column4"); } }

        public double Column5 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column5"); } }

        public double Column6 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column6"); } }

        public double Column7 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column7"); } }

        public double Column8 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column8"); } }

        public double Column9 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column9"); } }

        public double Column10 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column10"); } }

        public double Column11 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column11"); } }

        public double Column12 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column12"); } }

        public double Column13 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column13"); } }

        public double Column14 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column14"); } }

        public double Column15 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column15"); } }

        public double Column16 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column16"); } }

        public double Column17 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column17"); } }

        public double Column18 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column18"); } }

        public double Column19 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column19"); } }

        public double Column20 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column20"); } }

        public double Column21 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column21"); } }

        public double Column22 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column22"); } }

        public double Column23 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column23"); } }

        public double Column24 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column24"); } }

        public double Column25 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column25"); } }

        public double Column26 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column26"); } }

        public double Column27 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column27"); } }

        public double Column28 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column28"); } }

        public double Column29 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column29"); } }

        public double Column30 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column30"); } }

        public double Column31 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column31"); } }

        public double Column32 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column32"); } }

        public double Column33 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column33"); } }

        public double Column34 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column34"); } }

        public double Column35 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column35"); } }

        public double Column36 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column36"); } }

        public double Column37 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column37"); } }

        public double Column38 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column38"); } }

        public double Column39 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column39"); } }

        public double Column40 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column40"); } }

        public double Column41 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column41"); } }

        public double Column42 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column42"); } }

        public double Column43 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column43"); } }

        public double Column44 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column44"); } }

        public Complex Column45
        {
            get
            {
                return _complex;
            }
            set
            {
                if ( _complex != value )
                {
                    _complex = value;
                    RaisePropertyChanged("Column45");
                }
            }
        }
        private Complex _complex;

        public Int32 Column46
        {
            get
            {
                return column46;
            }
            set
            {
                column46 = value;
                RaisePropertyChanged("Column46");
            }
        }

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;
        private void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion
    }
}
